"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var webcheck_1 = require('webcheck');
var url = require('url');
var parser_1 = require('./parser');
var pkg = require('./package.json');
var emptyFilter = {
    test: function () {
        return true;
    }
};
var logError = function (err) {
    if (err) {
        console.error(err);
    }
};
var RobotsPlugin = (function (_super) {
    __extends(RobotsPlugin, _super);
    function RobotsPlugin(opts) {
        var _this = this;
        _super.call(this);
        this.package = pkg;
        opts = opts || {};
        opts.filterUrl = opts.filterUrl || emptyFilter;
        if (!opts.hasOwnProperty('sitemapLookup')) {
            opts.sitemapLookup = true;
        }
        if (!opts.hasOwnProperty('respectDelay')) {
            opts.respectDelay = true;
        }
        this.hosts = {};
        this.userAgent = opts.userAgent;
        this.middleware = function (result, next) {
            var parsedUrl, chunks = [];
            if (!opts.filterUrl.test(result.url)) {
                return next();
            }
            parsedUrl = url.parse(result.url);
            if (parsedUrl.path !== '/robots.txt') {
                if (!_this.hosts[parsedUrl.protocol + '//' + parsedUrl.host]) {
                    _this.handle.crawl({
                        url: parsedUrl.protocol + '//' + parsedUrl.host + '/robots.txt'
                    }, function (err) {
                        if (err) {
                            console.error(err);
                        }
                    });
                }
                return next();
            }
            if (result.response.statusCode !== 200) {
                _this.hosts[parsedUrl.protocol + '//' + parsedUrl.host] = {
                    allow: [{
                            test: function () {
                                return true;
                            }
                        }],
                    disallow: []
                };
                return next();
            }
            _this.userAgent = _this.userAgent || _this.handle.headers['User-Agent'];
            result.response.on('data', function (chunk) {
                chunks.push(chunk.toString());
            });
            result.response.on('end', function () {
                var thisUserAgent, allAgents, str, entry = {}, tmpSitemapUrl, sitemap;
                str = chunks.join('');
                thisUserAgent = str.split('User-Agent: ' + _this.userAgent);
                if (thisUserAgent.length !== 1) {
                    thisUserAgent = thisUserAgent[1].split('User-Agent:')[0];
                }
                else {
                    thisUserAgent = false;
                }
                allAgents = str.split('User-Agent: *');
                if (allAgents.length > 1) {
                    allAgents = allAgents[1].split('User-Agent:')[0];
                }
                else {
                    allAgents = false;
                }
                entry.sitemap = [];
                sitemap = str.split('Sitemap: ');
                if (sitemap.length > 1) {
                    for (var i = 1; i < sitemap.length; i += 1) {
                        tmpSitemapUrl = sitemap[i].split(/\r?\n/)[0].trim();
                        entry.sitemap.push(tmpSitemapUrl);
                        if (opts.sitemapLookup) {
                            _this.handle.crawl({
                                parameters: {
                                    sitemap: true
                                },
                                url: tmpSitemapUrl
                            }, logError);
                        }
                    }
                }
                if (allAgents) {
                    parser_1.parse(allAgents, entry);
                }
                if (thisUserAgent) {
                    parser_1.parse(thisUserAgent, entry);
                }
                _this.hosts[parsedUrl.protocol + '//' + parsedUrl.host] = entry;
            });
            return next();
        };
        this.on['crawl'] = function (settings) {
            var parsedUrl = url.parse(settings.url), entry;
            if (parsedUrl.path === '/robots.txt') {
                return;
            }
            entry = _this.hosts[parsedUrl.protocol + '//' + parsedUrl.host];
            if (!entry) {
                _this.handle.crawl({
                    immediately: true,
                    url: parsedUrl.protocol + '//' + parsedUrl.host + '/robots.txt'
                }, function (err) {
                    if (err) {
                        console.error(err);
                    }
                });
                return;
            }
            if (entry.delay && opts.respectDelay) {
                settings.wait = entry.delay;
            }
            for (var i = 0; i < entry.allow.length; i += 1) {
                if (entry.allow[i].test(parsedUrl.path)) {
                    return;
                }
            }
            for (var i = 0; i < entry.disallow.length; i += 1) {
                if (entry.disallow[i].test(parsedUrl.path)) {
                    settings.preventCrawl = true;
                    return;
                }
            }
        };
    }
    return RobotsPlugin;
}(webcheck_1.Plugin));
exports.RobotsPlugin = RobotsPlugin;
//# sourceMappingURL=webcheck-robots.js.map